package org.rcisoft.sys.user.dao;


import org.apache.ibatis.annotations.*;
import org.rcisoft.business.bdictionary.entity.BDictionary;
import org.rcisoft.business.bexternallink.dto.MenuDTO;
import org.rcisoft.business.blesson.dto.RoleTypeDepartDTO;
import org.rcisoft.core.base.BaseMapper;
import org.rcisoft.sys.user.bean.CurUser;
import org.rcisoft.sys.user.dto.MyInfoDTO;
import org.rcisoft.sys.user.dto.QueryMenuResDTO;
import org.rcisoft.sys.user.dto.QuerySysUserDTO;
import org.rcisoft.sys.user.dto.RoleDto;
import org.rcisoft.sys.user.entity.SysUser;
import org.rcisoft.sys.user.entity.UserRole;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * Created by lcy on 17/8/9.
 */
@Repository
public interface SysUserMapper extends BaseMapper<SysUser> {

    @Select("<script>SELECT DISTINCT su.* FROM s_user AS su " +
            "LEFT JOIN s_r_user_role AS ru ON su.business_id = ru.user_id " +
            "LEFT JOIN tm_admin_role AS sr ON sr.r_id = ru.role_id " +
            "WHERE su.del_flag = 0 " +
            "and su.corp_id = #{corpId} " +

            "<if test=\"name!=null and name != ''\"> and su.name like CONCAT('%',#{name},'%') </if>" +
            "<if test=\"roleName!=null and roleName != ''\"> and sr.r_name = #{roleName}</if>" +
            "<if test=\"userIds!=null and userIds.size() > 0 \"> " +
            " and su.business_id in <foreach item='item' index='index' collection='userIds' open='(' separator=',' close=')'> #{item} </foreach>" +
            "</if>" +
            "<if test=\"flag!=null and flag != ''\"> and su.flag = #{flag}</if></script>")
    @ResultMap(value = "BaseResultMap")
    List<SysUser> queryUsers(QuerySysUserDTO dto);

    @Select("<script>SELECT * " +
            "FROM s_user AS su " +
            "LEFT JOIN s_r_user_role AS ru ON su.business_id = ru.user_id " +
            "LEFT JOIN tm_admin_role AS sr ON ru.role_id = sr.r_id  " +
            "WHERE su.del_flag = 0  " +
            "and sr.r_status != 0 " +
            "AND su.business_id = #{businessId}</script>")
    @ResultMap(value = "RoleDtoMap")
    List<RoleDto> queryRoleByUserId(@Param("businessId") String businessId);

    @Select("<script>SELECT sr.r_id AS roleIdList  " +
            "FROM s_user AS su " +
            "LEFT JOIN s_r_user_role AS ru ON su.business_id = ru.user_id " +
            "LEFT JOIN tm_admin_role AS sr ON ru.role_id = sr.r_id  " +
            "WHERE su.del_flag = 0  " +
            "and sr.r_status != 0 " +
            "AND su.business_id = #{businessId}</script>")
    List<String> queryRoleIdByUserId(@Param("businessId") String businessId);

    @Update("<script> update s_user set flag = '1' where business_id in" +
            "<foreach item='item' index='index' collection='businessId' open='(' separator=',' close=')'> #{item} </foreach> " +
            "</script>")
    int startFlag(@Param("businessId") List<String> businessId);

    @Update("<script> update s_user set flag = '0' where business_id in" +
            "<foreach item='item' index='index' collection='businessId' open='(' separator=',' close=')'> #{item} </foreach> " +
            "</script>")
    int stopFlag(@Param("businessId") List<String> businessId);

    @Select("<script>select ur.user_id from s_r_user_role ur left JOIN tm_admin_role  sr on ur.role_id = sr.r_id where sr.r_type in" +
            "<foreach item='item' index='index' collection='rType' open='(' separator=',' close=')'> #{item} </foreach> " +
            "</script>")
    List<String> adminId(@Param("rType") String[] rType);

    @Select("<script> select * from s_r_user_role where user_id in" +
            " <foreach item='item' index='index' collection='userId' open='(' separator=',' close=')'> #{item} </foreach>" +
            "</script>")
    List<UserRole> queryUserRoleById(@Param("userId") List<String> userId);

    @Delete("<script> DELETE from  s_r_user_role where user_id in" +
            " <foreach item='item' index='index' collection='userId' open='(' separator=',' close=')'> #{item} </foreach>" +
            "</script>")
    int delUserRoleById(@Param("userId") List<String> userId);

    @Insert("<script> insert into s_r_user_role VALUES " +
            "<foreach collection=\"list\" item=\"item\" separator=\",\">" +
            "( #{item.businessId},#{item.userId},#{item.roleId})" +
            "</foreach></script>")
    int insertUserRoleById(List<UserRole> userRoles);

    @Select("<script>select * from s_user where login_name = #{username} and del_flag = 0 </script>")
    @ResultMap(value = "BaseResultMap")
    List<SysUser> queryUserByName(String username);

    @Insert("<script>INSERT INTO s_user" +
            "(create_date,update_date,create_by,update_by,remarks,del_flag,flag,business_id,login_name,password,name,sex,git_lab_id,git_lab_username,git_lab_impression_token,head_pic,email)VALUES" +
            "<foreach collection=\"list\" item=\"item\" separator=\",\">" +
            "( #{item.createDate},#{item.updateDate},#{item.createBy},#{item.updateBy},#{item.remarks},#{item.delFlag},#{item.flag},#{item.businessId},#{item.loginName},#{item.password},#{item.name},#{item.sex},#{item.gitLabId},#{item.gitLabUsername},#{item.gitLabImpressionToken},#{item.headPic},#{item.email})" +
            "</foreach></script>")
    int insertList(List<SysUser> users);

    @Insert("<script>INSERT INTO s_r_user_role" +
            "(business_id,user_id,role_id)VALUES" +
            "<foreach collection=\"list\" item=\"item\" separator=\",\">" +
            "(#{item.businessId},#{item.userId},#{item.roleId})" +
            "</foreach></script>")
    int insertUserRoleList(List<UserRole> users);

    @Insert("INSERT INTO s_r_user_role" +
            "(business_id,user_id,role_id)VALUES" +
            "(#{businessId},#{userId},#{roleId})")
    int insertUserRole(UserRole userRole);

    @Select("SELECT u.business_id from s_user u left join ${tab} s on u.login_name = s.code where s.business_id = #{id} and u.del_flag = 0")
    String selectUserIdByStuOrTeaId(@Param("id") String id, @Param("tab") String table);

    @Delete("delete from s_user where login_name = #{code}")
    int deleteByCode(String code);

    /**
     * 用户删除 cy 5-15
     *
     * @param code
     * @return
     */
    @Delete("DELETE s " +
            "FROM " +
            " s_user s " +
            "LEFT JOIN b_company c ON c. CODE = s.login_name " +
            "WHERE " +
            " c.business_id = #{code}")
    int deleteByCompany(String code);


    @Select("<script>select * from s_user where business_id = #{userId} and password = #{password} limit 1</script>")
    @ResultMap(value = "BaseResultMap")
    SysUser checkPassword(@Param("userId") String userId, @Param("password") String password);

    @Update("update s_user set password =#{password} where business_id = '${userId}'")
    int changePassword(@Param("userId") String userId, @Param("password") String password);

    @Select("<script>select password from s_user where business_id = #{userId} </script>")
    String queryPasswordById(String userId);

    @Update("<script>update s_user set password = #{password} where 1=1 " +
            "and login_name = #{loginName} " +
            "and del_flag = 0</script>")
    int resetPassword(@Param("loginName") String loginNam, @Param("password") String password);

    @Select("<script>select business_id from s_user where login_name = #{studentCode} and del_flag = 0 and flag = 1</script>")
    String queryUserIdByLoginName(String studentCode);

    @Delete("<script>delete from s_r_user_role where user_id = #{businessId}</script>")
    int deleteUserRole(String businessId);

    @Select("<script>select su.*,ba.`name` as agencyName,sr.r_name as roleName,sr.business_id as roleId from s_user su " +
            "LEFT JOIN s_r_user_role sru on su.business_id = sru.user_id " +
            "LEFT JOIN tm_admin_role sr on sru.role_id = sr.r_id " +
            "where su.is_created = '1' and su.del_flag = 0 and su.flag = 1  " +
            "</script>")
    @ResultMap(value = "BaseResultMap")
    List<SysUser> queryCreatedUsers(SysUser sysUser);

    @Select("<script>select agency_code from s_user where business_id = #{businessId}</script>")
    String queryAgencyCodeByBusinessId(String businessId);


    @Update("update s_user set flag =#{flag} where login_name = #{loginName} and del_flag = 0 and flag = 0")
    int stopOrStartUser(SysUser user);

    @Update("update s_user set flag =#{flag} where business_id = #{businessId} ")
    int stopOrStartUserByUserBusinessId(SysUser sysUser);

    @Select("select business_id from s_user where login_name = #{0} and del_flag = 0")
    String selectByLoginName(String loginName);

    @Select("select count(*) from s_r_user_role " +
            "left join s_user on s_user.business_id = s_r_user_role.user_id " +
            "where role_id = #{roleId} and s_user.del_flag = 0 ")
    int countUserByRole(String roleId);


    int deleteTeacherUserRole(List<SysUser> userTeacherList);

    int deleteCompanyUserRole(List<SysUser> userCompanyList);

    @Select("<script>SELECT sr.type AS type,sr.depart_ids departs " +
            "FROM s_user AS su " +
            "LEFT JOIN s_r_user_role AS ru ON su.business_id = ru.user_id " +
            "LEFT JOIN tm_admin_role AS sr ON ru.role_id = sr.r_id  " +
            "WHERE su.del_flag = 0  " +
            "and sr.r_status !=0 " +
            "AND su.business_id = #{userId}</script>")
    List<RoleTypeDepartDTO> queryRoleTypeByUserId(@Param("userId")String userId);

    /**
     * 查询用户 menu 19-09-30
     * @param corpId
     * @param businessId
     * @param modelId
     * @return
     */
    @Select("SELECT DISTINCT ur.user_id, rm.MENU_ID menu_id, sm.`NAME` menu_name, sm.PARENT_ID, sm.HREF href  " +
            "FROM s_user su " +
            "LEFT JOIN s_r_user_role ur ON ur.user_id = su.business_id " +
            "LEFT JOIN s_role_menu rm ON ur.role_id = rm.role_id " +
            "LEFT JOIN s_menu sm ON sm.business_id = rm.MENU_ID  " +
            "WHERE " +
            "su.del_flag != 1 " +
            "AND su.flag = 1 " +
            "and sm.del_flag != 1 " +
            "and sm.flag = 1 " +
            "AND su.corp_id = #{corpId} " +
            "AND su.business_id = #{businessId} " +
            "AND sm.MODEL_ID = #{modelId} " +
            "ORDER BY sm.BUSINESS_ID ")
    @ResultMap(value = "DtoResultMap")
    List<QueryMenuResDTO> queryMenus(@Param("corpId") String corpId, @Param("businessId") String businessId, @Param("modelId") String modelId );

    @Select("select * from s_user where business_id = #{userId}")
    @ResultMap(value = "BaseResultMap")
    SysUser queryUserState(@Param("userId")String userId);

    @Select("<script>select business_id id, `name` name,`value` value  from s_user where business_id = #{userId}" +
            "</script>")
    MyInfoDTO queryMyInfo(CurUser curUser);

    @Select("<script>select business_id id,`name` name ,head_pic headPic, `value` value, lock_num lockNum from s_user where business_id = #{id}" +
            "</script>")
    MyInfoDTO getNameById(String id);

    @Select(" select r.r_privilege " +
            " from tm_admin_role r " +
            " left join s_r_user_role ur on ur.role_id = r.r_id " +
            " left join s_user u  on ur.user_id = u.business_id " +
            " where 1=1 " +
            " and u.business_id = #{userId} " +
            " and u.del_flag !=1 " +
            " and u.corp_id = #{corpId} " +
            " and r.corp_id = #{corpId} " +
            " and u.flag = 1 " +
            " and r.r_status = 1 ")
    List<String>queryAllRoleById(CurUser curUser);

    @Select("<script>select " +
            " (select count(1) from " +
            " b_lesson_person blp left join b_lesson bl on blp.lesson_id = bl.business_id " +
            " where blp.del_flag != 1 AND blp.flag = 1 " +
            " and bl.del_flag != 1 AND bl.flag = 1 " +
            " and blp.is_finish = 1 " +
//            " and bl.release_state = 2 " +
            " and bl.lesson_type = 0 " +
            " and bl.corp_id = #{corpId} " +
            " and blp.person_id = #{userId} and blp.is_finish is not null) lessonCount, " +

            " (select count(1) from " +
            " b_lesson_person blp left join b_lesson bl on blp.lesson_id = bl.business_id " +
            " where blp.del_flag != 1 AND blp.flag = 1 " +
            " and bl.del_flag != 1 AND bl.flag = 1 " +
//            " and bl.release_state = 2 " +
            " and blp.train_is_sign = 0" +
            " and blp.person_id = #{userId}) trainCount, " +

            "(SELECT (SELECT count(0) FROM tm_paper tp left join tm_paper_link_branch tpb on tpb.ln_pid = tp.p_id " +
            "  where 1=1 and tpb.ln_buid = #{userId} ) -  " +
            "  (select count(0) from tm_examdata where e_uid = #{userId} " +
            "and e_pid in (SELECT p_id FROM tm_paper tp left join tm_paper_link_branch tpb on tpb.ln_pid = tp.p_id " +
            "  where 1=1 and tpb.ln_buid = #{userId} ))) as notFinishedExam " +
            "</script>")
    MyInfoDTO queryMyLessonCount(CurUser curUser);

    /**
     * 初始化多条数据到Bdictionary表
     * @param list
     * @return
     */
    @Insert("<script> INSERT INTO b_dictionary " +
            "(BUSINESS_ID,PID,TYPE,NAME,`VALUE`,CREATE_BY,CREATE_DATE,UPDATE_BY,UPDATE_DATE,DEL_FLAG,FLAG,REMARKS,corp_id,sort) VALUES" +
            "<foreach collection=\"list\" item=\"item\" separator=\",\">" +
            "(#{item.businessId},#{item.pid},#{item.type},#{item.name},#{item.value},#{item.createBy},#{item.createDate},#{item.updateBy},#{item.updateDate},#{item.delFlag},#{item.flag},#{item.remarks},#{item.corpId},#{item.sort})"+
            "</foreach> </script>")
    int initDictionary(List<BDictionary> list);


    @Select("<script>SELECT * FROM s_user  " +
            "WHERE 1=1 " +
            " and del_flag = 0 " +
            " and flag = 1 " +
            " and business_id in  "+
            "<foreach item='item' index='index' collection='ids' open='(' separator=',' close=')'> #{item} </foreach> " +
            "</script>")
    @ResultMap(value = "BaseResultMap")
    List<SysUser> selectByIds(@Param("ids") List<String> ids);

    @Select("<script>SELECT * FROM s_user  " +
            "WHERE 1=1 " +
            " and business_id in  "+
            "<foreach item='item' index='index' collection='ids' open='(' separator=',' close=')'> #{item} </foreach> " +
            "</script>")
    @ResultMap(value = "BaseResultMap")
    List<SysUser> selectAppointPersonByIds(@Param("ids") List<String> ids);


    /**
     * 根据accountId获取businessId
     * @param accountId
     * @return
     */
    @Select("select * from s_user where account_id in(${accountId}) and corp_id = #{corpId}")
    @ResultMap(value = "BaseResultMap")
    List<SysUser> getBusinessIdByAccountId(@Param("accountId") String accountId ,@Param("corpId")String corpId);


    @Select("select count(0) from s_user where del_flag != 1 and flag = 1 and corp_id = #{corpId} and business_id = #{userId}")
    int getUserflag (CurUser curUser);


    @Select("<script>select title menuName,external_url url,mobile_logo mobileLogo ,computer_logo computerLogo  ,type " +
            "from b_external_link where 1=1  " +
            "<if test=\"type !=null \"> and type in ('1',${type}) </if>" +
            "  and corp_id = #{corpId} " +
            " and del_flag !=1 and flag = 1 ORDER BY sort </script>")
    List<MenuDTO> getReadingMenu(@Param("type") String type, @Param("corpId") String corpId);

}
